Package: devtools
Title: Tools to Make Developing R Packages Easier
Version: 2.5.2
Authors@R: c(
    person("Hadley", "Wickham", role = "aut"),
    person("Jim", "Hester", role = "aut"),
    person("Winston", "Chang", role = "aut"),
    person("Jennifer", "Bryan", , "jenny@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6983-2759")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Collection of package development tools.
License: MIT + file LICENSE
URL: https://devtools.r-lib.org/, https://github.com/r-lib/devtools
BugReports: https://github.com/r-lib/devtools/issues
Depends: R (>= 4.1), usethis (>= 3.2.1)
Imports: cli (>= 3.6.6), desc (>= 1.4.3), ellipsis (>= 0.3.3), fs (>=
        2.1.0), lifecycle (>= 1.0.5), memoise (>= 2.0.1), miniUI (>=
        0.1.2), pak (>= 0.9.5), pkgbuild (>= 1.4.8), pkgdown (>=
        2.2.0), pkgload (>= 1.5.2), profvis (>= 0.4.0), rcmdcheck (>=
        1.4.0), rlang (>= 1.2.0), roxygen2 (>= 7.3.3), rversions (>=
        3.0.0), sessioninfo (>= 1.2.3), testthat (>= 3.3.2), urlchecker
        (>= 1.0.1), withr (>= 3.0.2)
Suggests: BiocManager (>= 1.30.18), callr (>= 3.7.1), covr (>= 3.5.1),
        curl (>= 4.3.2), digest (>= 0.6.29), DT (>= 0.23), foghorn (>=
        1.4.2), gh (>= 1.3.0), httr2 (>= 1.0.0), knitr (>= 1.39), lintr
        (>= 3.0.0), quarto (>= 1.5.1), remotes (>= 2.5.0), rmarkdown
        (>= 2.14), rstudioapi (>= 0.13), spelling (>= 2.2), xml2
VignetteBuilder: knitr, quarto
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-29 15:42:59 UTC; jenny
Author: Hadley Wickham [aut],
  Jim Hester [aut],
  Winston Chang [aut],
  Jennifer Bryan [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6983-2759>),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Jennifer Bryan <jenny@posit.co>
Repository: CRAN
Date/Publication: 2026-04-30 07:40:02 UTC
Built: R 4.6.0; ; 2026-05-01 22:40:30 UTC; unix
