Package: testError
Title: Test package to check error message gives file/line info
License: GPL-2
Description:
Author: Hadley <h.wickham@gmail.com>
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1