Package: testHelp
Title: Tools to make developing R code easier
License: GPL-2
Description: Test package for devtools help.
Author: Hadley <h.wickham@gmail.com>
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1
RoxygenNote: 6.0.1
