Package: testTest
Title: Tools to Make Developing R Code Easier
License: GPL-2
Description: Package description.
Authors@R:
    c(person(given = "Hadley",
             family = "Wickham",
             role = c("aut", "cre"),
             email = "h.wickham@gmail.com"))
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1
Suggests:
    testthat
RoxygenNote: 5.0.1
