Package: testS4union
Title: Test package for S4 class unions
License: GPL-2
Description: 
Author: Winston Chang <winston@stdout.org>
Maintainer: Winston Chang <winston@stdout.org>
Version: 0.1
Collate:
    'classes.r'
Imports:
    stats4,
    methods
