Package: infrastructure
Title: What the Package Does (one line, title case)
Version: 0.0.0.9000
Authors@R: person("First", "Last", email = "first.last@example.com", role = c("aut", "cre"))
Description: What the package does (one paragraph).
Depends: R (>= 3.3.1)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Suggests: testthat,
    knitr,
    rmarkdown,
    covr
VignetteBuilder: knitr
LinkingTo: Rcpp
Imports: Rcpp
