Package: testDllLoad
Title: Test package for loading and unloading DLLs
License: GPL-2
Description:
Author: Hadley <h.wickham@gmail.com>
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1
Collate: a.r