Package: testS4sort
Title: Test package for sorting S4 classes
License: GPL (>= 2)
Description:
Author: Facundo Muñoz <facundo.munoz@orleans.inra.fr>
Maintainer: Facundo Muñoz <facundo.munoz@orleans.inra.fr>
Version: 0.1
Collate:
    'classes.r'
Imports:
    methods
