# Generated by roxygen2: do not edit by hand

S3method(print,dev_sitrep)
S3method(print,maintainers)
export(as.package)
export(bash)
export(build)
export(build_manual)
export(build_readme)
export(build_rmd)
export(build_site)
export(build_vignettes)
export(check)
export(check_built)
export(check_dep_version)
export(check_doc_fields)
export(check_mac_devel)
export(check_mac_release)
export(check_man)
export(check_rhub)
export(check_win_devel)
export(check_win_oldrelease)
export(check_win_release)
export(clean_dll)
export(clean_vignettes)
export(create)
export(dev_mode)
export(dev_package_deps)
export(dev_packages)
export(dev_sitrep)
export(document)
export(find_rtools)
export(github_pull)
export(github_release)
export(has_devel)
export(has_tests)
export(install)
export(install_bioc)
export(install_bitbucket)
export(install_cran)
export(install_deps)
export(install_dev)
export(install_dev_deps)
export(install_git)
export(install_github)
export(install_gitlab)
export(install_local)
export(install_svn)
export(install_url)
export(install_version)
export(is.package)
export(is_loading)
export(lint)
export(load_all)
export(loaded_packages)
export(missing_s3)
export(package_file)
export(package_info)
export(parse_deps)
export(r_env_vars)
export(release)
export(release_checks)
export(reload)
export(revdep)
export(revdep_maintainers)
export(run_examples)
export(session_info)
export(show_news)
export(source_gist)
export(source_url)
export(spell_check)
export(submit_cran)
export(test)
export(test_active_file)
export(test_coverage)
export(test_coverage_active_file)
export(test_coverage_file)
export(test_file)
export(uninstall)
export(unload)
export(update_packages)
export(uses_testthat)
export(wd)
export(with_debug)
import(fs)
import(rlang)
importFrom(ellipsis,check_dots_used)
importFrom(lifecycle,deprecated)
importFrom(memoise,memoise)
importFrom(pkgbuild,clean_dll)
importFrom(pkgbuild,find_rtools)
importFrom(pkgbuild,has_devel)
importFrom(pkgbuild,with_debug)
importFrom(pkgload,check_dep_version)
importFrom(pkgload,is_loading)
importFrom(pkgload,parse_deps)
importFrom(pkgload,unload)
importFrom(sessioninfo,package_info)
importFrom(sessioninfo,session_info)
importFrom(stats,update)
importFrom(usethis,use_test)
