#' Test function for help
#'
#' The purpose of this function is to test out \code{help} and \code{?} from
#' devtools.
#'
#' @examples
#' stopifnot(foofoo() == 'You called foofoo.')
#' @export
foofoo <- function() "You called foofoo."
