#!/hint/bash
# License: Unspecified
:

# shellcheck disable=2034
_arch=(
	x86_64
	any
)

# shellcheck disable=2034
_tags=(
	core-x86_64 core-any
	extra-x86_64 extra-any
	multilib-x86_64
	staging-x86_64 staging-any
	testing-x86_64 testing-any
	multilib-testing-x86_64
	multilib-staging-x86_64
	community-x86_64 community-any
	community-staging-x86_64 community-staging-any
	community-testing-x86_64 community-testing-any
	kde-unstable-x86_64 kde-unstable-any
	gnome-unstable-x86_64 gnome-unstable-any
)
