# github_ functions are deprecated

    Code
      . <- github_pull(1)
    Condition
      Warning:
      `github_pull()` was deprecated in devtools 2.5.0.
      i Please use `remotes::github_pull()` instead.
    Code
      . <- github_release()
    Condition
      Warning:
      `github_release()` was deprecated in devtools 2.5.0.
      i Please use `remotes::github_release()` instead.

