# load_all() errors if called recursively

    Code
      load_all()
    Condition
      Error in `load_all()`:
      ! Recursive loading detected.
      i Did you accidentally include `load_all()` in an R source file?

