# check_for_rstudio_updates

    Code
      writeLines(check_for_rstudio_updates("windows", "haha-no-wut", TRUE))
    Output
      Unable to check for RStudio updates (you're using haha-no-wut).

---

    Code
      writeLines(check_for_rstudio_updates("darwin", "0.0.1", TRUE))
    Output
      RStudio {VERSION} is now available (you're using 0.0.1).
      Download at: https://www.rstudio.com/products/rstudio/download/

---

    Code
      writeLines(check_for_rstudio_updates("windows", "1.4.1717", TRUE))
    Output
      RStudio {VERSION} is now available (you're using 1.4.1717).
      Download at: https://www.rstudio.com/products/rstudio/download/

---

    Code
      writeLines(check_for_rstudio_updates("darwin", "2021.09.1+372", TRUE))
    Output
      RStudio {VERSION} is now available (you're using 2021.09.1+372).
      Download at: https://www.rstudio.com/products/rstudio/download/

---

    Code
      writeLines(check_for_rstudio_updates("windows", "2021.09.0-daily+328", TRUE))
    Output
      RStudio {VERSION} is now available (you're using 2021.09.0-daily+328).
      Download at: https://www.rstudio.com/products/rstudio/download/

