# build_vignettes() and clean_vignettes() are deprecated

    Code
      build_vignettes(pkg)
    Condition
      Warning:
      `build_vignettes()` was deprecated in devtools 2.5.0.
    Code
      clean_vignettes(pkg)
    Condition
      Warning:
      `clean_vignettes()` was deprecated in devtools 2.5.0.

