# build_manual() shows stderr on failure

    Code
      build_manual(pkg)
    Message
      Saving manual to '<pkgdir>_0.0.0.9000.pdf'
    Condition
      Error in `build_manual()`:
      ! Failed to build manual.
      ! LaTeX Error: File `inconsolata.sty' not found.
      Caused by error in `rd2pdf()`:
      ! System command 'R' failed

