# release() is deprecated

    Code
      . <- release()
    Condition
      Warning:
      `release()` was deprecated in devtools 2.5.0.
      i Please use `usethis::use_release_issue()` instead.
      Error in `release()`:
      ! Interactive session required.

