% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.r
\name{is.package}
\alias{is.package}
\title{Is the object a package?}
\usage{
is.package(x)
}
\description{
Is the object a package?
}
\keyword{internal}
