% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/has-tests.R
\name{has_tests}
\alias{has_tests}
\title{Was devtools installed with tests?}
\usage{
has_tests()
}
\description{
Was devtools installed with tests?
}
\keyword{internal}
