% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.R
\name{uses_testthat}
\alias{uses_testthat}
\title{Does a package use testthat?}
\usage{
uses_testthat(pkg = ".")
}
\description{
Does a package use testthat?
}
\keyword{internal}
